package it.amattioli.dominate.hibernate;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.Predicate;
import org.hibernate.Query;

import it.amattioli.dominate.Entity;
import it.amattioli.dominate.Filter;
import it.amattioli.dominate.Repository;
import it.amattioli.dominate.Specification;
import it.amattioli.dominate.specifications.hibernate.HqlAssembler;

public class CollectionHibernateRepository<I extends Serializable,T extends Entity<I>> extends AbstractHibernateRepository<I,T> {
	private Collection<T> content;

    public CollectionHibernateRepository(Collection<T> content) {
        this.content = content;
    }
    
    public T get(I id) {
    	for (T curr : content) {
            if (curr.getId().equals(id)) {
                return curr;
            }
        }
        return null;
    }

    public void put(T object) {
        content.add(object);
    }

    public void refresh(I objectId) {
		// Non fa nulla, il contenuto del repository  fissato in fase di 
    	// costruzione e non pu essere riletto
	}

	public void refresh(T object) {
		// Non fa nulla, il contenuto del repository  fissato in fase di 
    	// costruzione e non pu essere riletto
	}

	@Override
	public boolean isRemoveAllowed() {
		return true;
	}

	@Override
	public void remove(I objectId) {
		remove(get(objectId));
	}

	@Override
	public void remove(T object) {
		content.remove(object);
	}

	@SuppressWarnings("unchecked")
	public List<T> list() {
        String hqlQuery = "";
        if (getOrderProperty() != null) {
            hqlQuery = hqlQuery + getHqlOrder();
        }
        Query query = getHibernateSession().createFilter(content,hqlQuery);
        if (getFirst() != 0) {
        	query.setFirstResult(getFirst());
        }
        if (getLast() != 0) {
        	query.setMaxResults(getLast());
        }
        List<T> result = query.list();
        return result;
    }

	@SuppressWarnings("unchecked")
    public List<T> list(Filter criteria) {
        List<T> result;
        HibernateFilter hCriteria = (HibernateFilter)criteria;
        String hqlQuery = hCriteria.getHqlFilter();
        Predicate additional = hCriteria.getCollectionPredicate();
        int first = criteria.getFirst() == 0 ? getFirst() : criteria.getFirst();
        int last = criteria.getLast() == 0 ? getLast() : criteria.getLast();
        if (hqlQuery != null) {
            if (getOrderProperty() != null) {
                hqlQuery = hqlQuery + " " + getHqlOrder();
            }
			Query query = getHibernateSession().createFilter(content,hqlQuery);
			hCriteria.setHqlParameters(query);
			if (additional == null) {
			    if (first != 0) {
			    	query.setFirstResult(first);
			    }
			    if (last != 0) {
			    	query.setMaxResults(last);
			    }
			}
			result = query.list();
        } else {
            result = new ArrayList<T>(content);
        }
        if (additional != null) {
            CollectionUtils.filter(result,additional);
            result = result.subList(first, last == 0 ? result.size() : last);
        }
        return result;
    }
	
	@Override
	public List<T> list(Specification<T> spec) {
    	List<T> result;
    	//int first = criteria.getFirst() == 0 ? getFirst() : criteria.getFirst();
    	int first = getFirst();
        //int last = criteria.getLast() == 0 ? getLast() : criteria.getLast();
    	int last = getLast();
		HqlAssembler hAssembler = new HqlAssembler("", getOrders(), false);
    	if (spec.supportsAssembler(hAssembler)) {
    		spec.assembleQuery(hAssembler);
    		Query query = hAssembler.assembledHqlFilter(getHibernateSession(), content);
    		if (first != 0) {
                query.setFirstResult(first);
            }
            if (last != 0) {
                query.setMaxResults(last);
            }
            result = query.list();
    	} else {
    		throw new IllegalArgumentException("Unsupported specification");
    	}
		return result;
	}

	public <J extends Serializable, E extends Entity<J>> Repository<J, E> getDetailRepository(Collection<E> details) {
		CollectionHibernateRepository<J,E> rep = new CollectionHibernateRepository<J,E>(details);
		rep.setSessionManager(getSessionManager());
		return rep;
	}
	
	public void fillCollection(Filter criteria, Collection<? super T>... toBeFilled) {
		//TODO: andrebbe implementato usando il metodo scroll di Query e Criteria
		Collection<T> result = list(criteria);
		for (T currRes: result) {
			for(Collection<? super T> currColl: toBeFilled) {
				currColl.add(currRes);
			}
		}
	}

	public void fillCollection(Collection<? super T>... toBeFilled) {
		//TODO: andrebbe implementato usando il metodo scroll di Query e Criteria
		Collection<T> result = list();
		for (T currRes: result) {
			for(Collection<? super T> currColl: toBeFilled) {
				currColl.add(currRes);
			}
		}
	}

}
