package it.amattioli.dominate.hibernate.filters;

import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.Collection;

import org.hibernate.Criteria;
import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.criterion.Restrictions;
import org.hibernate.usertype.UserType;

public class EnumFilter<T extends Enum<T>> implements ValueListAttributeFilter<T> {
    private T value;
    private String alias;
    private String attributeName;
    private Class<T> enumClass;
    private Class<? extends UserType> hibernateType;

    public EnumFilter(String attributeName, Class<T> enumClass, Class<? extends UserType> hibernateType) {
        this.attributeName = attributeName;
        this.enumClass = enumClass;
        this.hibernateType = hibernateType;
    }
    
    public EnumFilter(String alias, String attributeName, Class<T> enumClass, Class<? extends UserType> hibernateType) {
        this(attributeName, enumClass, hibernateType);
        this.alias = alias;
    }

    public T get() {
        return value;
    }

    public void set(T value) {
        this.value = value;
    }

    private boolean wasSet() {
        return get() != null;
    }

    @SuppressWarnings("unchecked")
    public Collection<T> getValues() {
        try {
            return Arrays.asList((T[])enumClass.getMethod("values").invoke(null));
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new RuntimeException(e);
        } catch (NoSuchMethodException e) {
            throw new RuntimeException(e);
        }
    }

    public void addCriteria(Criteria crit) {
        crit.add(Restrictions.eq(attributeName, get()));
    }

    public void addHqlCondition(StringBuffer buffer) {
        if (wasSet()) {
            if (buffer.length() == 0) {
                buffer.append("where ");
            } else {
                buffer.append("and ");
            }
            if (alias != null) {
            	buffer.append(alias).append('.');
            }
            buffer.append(attributeName)
                  .append(" like :")
                  .append(attributeName)
                  .append(" ");
        }
    }

    public void setHqlParam(Query query) {
        if (wasSet()) {
            query.setParameter(attributeName, get(), Hibernate.custom(hibernateType));
        }
    }

}
