package it.amattioli.dominate.hibernate.filters;


import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;

import org.hibernate.Criteria;
import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.criterion.Restrictions;
import org.hibernate.usertype.UserType;

public class MultipleEnumFilter<T extends Enum<T>> implements HibernateAttributeFilter<Collection<T>> {
	private String alias;
    private Collection<T> values;
    private String attributeName;
    private Class<T> enumClass;
    private Class<? extends UserType> hibernateType;

    public MultipleEnumFilter(String attributeName, Class<T> enumClass, Class<? extends UserType> hibernateType) {
        this.attributeName = attributeName;
        this.enumClass = enumClass;
        this.hibernateType = hibernateType;
    }
    
    public MultipleEnumFilter(String alias, String attributeName, Class<T> enumClass, Class<? extends UserType> hibernateType) {
    	this(attributeName, enumClass, hibernateType);
    	this.alias = alias;
    }

    public Collection<T> get() {
        return values;
    }

    public void set(Collection<T> values) {
        this.values = new ArrayList<T>(values);
    }

    private boolean wasSet() {
        return get() != null && !get().isEmpty();
    }

    @SuppressWarnings("unchecked")
    public Collection<T> getValues() {
        try {
            return Arrays.asList((T[])enumClass.getMethod("values").invoke(null));
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new RuntimeException(e);
        } catch (NoSuchMethodException e) {
            throw new RuntimeException(e);
        }
    }

    public void addCriteria(Criteria crit) {
        if (wasSet()) {
            crit.add(Restrictions.in(attributeName, get()));
        }
    }

    public void addHqlCondition(StringBuffer buffer) {
        if (wasSet()) {
            if (buffer.length() == 0) {
                buffer.append("where ");
            } else {
                buffer.append("and ");
            }
            if (alias != null) {
            	buffer.append(alias).append('.');
            }
            buffer.append(attributeName)
                  .append(" in ( ");
            for (int i = 0; i < get().size(); i++) {
                buffer.append(":").append(attributeName.replace('.', '_')).append(i);
                if (i != get().size() - 1) {
                    buffer.append(" , ");
                }
            }
            buffer.append(" )");
        }
    }

    public void setHqlParam(Query query) {
        if (wasSet()) {
            int i = 0;
            for (T value : get()) {
                query.setParameter(attributeName.replace('.', '_') + (i++), value, Hibernate.custom(hibernateType));
            }
        }
    }

}
