package it.amattioli.dominate.hibernate.filters;


import java.util.Arrays;
import java.util.Collection;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.criterion.Restrictions;

public class StringFilter implements HibernateAttributeFilter<String> {
	private String alias;
    private String attributeName;
    private String value;
    private ComparisonType filterType = ComparisonType.STARTS;

    public StringFilter(String attributeName) {
        this.attributeName = attributeName;
    }
    
    public StringFilter(String alias, String attributeName) {
    	this.alias = alias;
        this.attributeName = attributeName;
    }

    public String get() {
        return value;
    }

    public void set(String value) {
        this.value = value;
    }

    public ComparisonType getFilterType() {
        return filterType;
    }

    public void setFilterType(ComparisonType filterType) {
        this.filterType = filterType;
    }

    public void setFilterType(String filterType) {
        setFilterType(ComparisonType.valueOf(filterType));
    }

    public Collection<ComparisonType> getFilterTypeValues() {
        return Arrays.asList(ComparisonType.values());
    }

    protected boolean wasSet() {
        return get() != null && !get().equals("");
    }

    public void addCriteria(Criteria crit) {
        if (wasSet()) {
            crit.add(Restrictions.ilike(attributeName, get(), getFilterType().getMatchMode()));
        }
    }

    public void addHqlCondition(StringBuffer buffer) {
        if (wasSet()) {
            if (buffer.length() == 0) {
                buffer.append("where ");
            } else {
                buffer.append("and ");
            }
            if (alias != null) {
            	buffer.append(alias).append('.');
            }
            buffer.append(attributeName)
                  .append(" like :")
                  .append(attributeName.replace('.', '_'))
                  .append(' ');
        }
    }

    public void setHqlParam(Query query) {
        if (wasSet()) {
        	String param = get();
        	if (ComparisonType.CONTAINS.equals(getFilterType())) {
        		param = "%"+param;
        	}
        	if (ComparisonType.CONTAINS.equals(getFilterType()) || ComparisonType.STARTS.equals(getFilterType())) {
        		param = param+"%";
        	}
            query.setParameter(attributeName.replace('.', '_'), param);
        }
    }

}
