package it.amattioli.dominate.specifications;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;

import it.amattioli.dominate.Entity;
import it.amattioli.dominate.Specification;

public abstract class CompositeSpecification<T extends Entity<?>> extends AbstractSpecification<T> {
	private Collection<Specification<T>> specifications = new ArrayList<Specification<T>>();

	public CompositeSpecification() {
		
	}
	
	public CompositeSpecification(Specification<T>... specs) {
		specifications.addAll(Arrays.asList(specs));
	}
	
	public void addSpecification(Specification<T> spec) {
		specifications.add(spec);
	}
	
	public void removeSpecification(Specification<T> spec) {
		specifications.remove(spec);
	}

	protected Collection<Specification<T>> getSpecifications() {
		return specifications;
	}
	
	@Override
	public boolean supportsAssembler(Assembler assembler) {
		for (Specification<T> spec : specifications) {
			if (!spec.supportsAssembler(assembler)) {
				return false;
			}
		}
		return true;
	}
	
	@Override
	public boolean wasSet() {
		for (Specification<T> spec : specifications) {
			if (spec.wasSet()) {
				return true;
			}
		}
		return false;
	}
	
}
