package it.amattioli.dominate.specifications.beans;

import java.beans.PropertyDescriptor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.beanutils.BasicDynaClass;
import org.apache.commons.beanutils.DynaBean;
import org.apache.commons.beanutils.DynaClass;
import org.apache.commons.beanutils.DynaProperty;
import org.apache.commons.beanutils.PropertyUtils;

import it.amattioli.dominate.Entity;
import it.amattioli.dominate.Specification;
import it.amattioli.dominate.properties.Properties;
import it.amattioli.dominate.specifications.ConjunctionSpecification;

public class BeanSpecification<T extends Entity<?>> extends ConjunctionSpecification<T> implements DynaBean {
	private static ArrayList<PropertySpecificationFactory> factories = new ArrayList<PropertySpecificationFactory>();
	static {
		registerFactory(new ObjectPropertySpecificationFactory());
		registerFactory(new EntityPropertySpecificationFactory());
		registerFactory(new EnumPropertySpecificationFactory());
		registerFactory(new StringPropertySpecificationFactory());
	};
	
	public static void registerFactory(PropertySpecificationFactory factory) {
		factories.add(0,factory);
	}
	
	private Map<String, Specification<T>> specs = new HashMap<String, Specification<T>>();
	private DynaClass dynaClass;
	
	public BeanSpecification(DynaClass dynaClass) {
		this.dynaClass = dynaClass;
	}
	
	public BeanSpecification(Class<T> beanClass) {
		List<DynaProperty> props = new ArrayList<DynaProperty>();
		for (PropertyDescriptor descriptor: PropertyUtils.getPropertyDescriptors(beanClass)) {
			Specification<T> spec = null;
			for (PropertySpecificationFactory factory: factories) {
				if (factory.canBuildSpecificationFor(descriptor)) {
					spec = factory.newSpecification(descriptor);
					break;
				}
			}
			addSpecification(descriptor.getName(), spec);
			props.add(new DynaProperty(descriptor.getName(), descriptor.getPropertyType()));
		}
		dynaClass = new BasicDynaClass(beanClass.getName()+"Specification", 
				                       BeanSpecification.class, 
				                       props.toArray(new DynaProperty[props.size()]));
	}
	
	private void addSpecification(String propertyName, Specification<T> spec) {
		specs.put(propertyName, spec);
		addSpecification(spec);
	}

	@Override
	public boolean contains(String name, String key) {
		throw new UnsupportedOperationException();
	}

	@Override
	public Object get(String arg0, int arg1) {
		throw new UnsupportedOperationException();
	}

	@Override
	public Object get(String arg0, String arg1) {
		throw new UnsupportedOperationException();
	}

	@Override
	public Object get(String propertyName) {
		return Properties.get(specs.get(propertyName),"value");
	}

	@Override
	public DynaClass getDynaClass() {
		return dynaClass;
	}

	@Override
	public void remove(String arg0, String arg1) {
		throw new UnsupportedOperationException();
	}

	@Override
	public void set(String arg0, int arg1, Object arg2) {
		throw new UnsupportedOperationException();
	}

	@Override
	public void set(String propertyName, Object specValue) {
		Properties.set(specs.get(propertyName), "value", specValue);
	}

	@Override
	public void set(String arg0, String arg1, Object arg2) {
		throw new UnsupportedOperationException();
	}
	
}
